--[[

   monsters.lua
   Handles the monsters in the platform game of Because it's fun, Fay.
	
   Copyright 2008 Oxeye Game Studio. http://www.oxeyegames.com
	
   File contributors:
      Jens Bergensten
      Daniel Brynolf

   This game was written for the TIGSource Procedural Generation
   competition in May 2008. You may use this Lua source code in
   any way you please, we don't mind. However, the game's sound
   effects and artwork belong to us, so please ask for permission
   before you do anything funny =)

]]

local gameField = nil
local playerFay = nil
local playerCorn = nil
local cornShots = nil
local powerups = nil

local monsters = {}
local monsterKinds = {}

local monsterSprite = {}

local function addMonsterKind(startHealth, updateMethod, render, attackMethod, deathMethod, powerUpChance)
	table.insert(monsterKinds, {
			startHealth = startHealth,
			updateMethod = updateMethod,
			attackMethod = attackMethod,
			render = render,
			deathMethod = deathMethod,
			powerUpChance = powerUpChance
		})
end

local function createMonster(kind, x, y)
	table.insert(monsters, {
			x=x, y=y, kind=kind,
			vx = 0, vy = 0,
			active = false,
			sprite = kind.startSprite,
			health = kind.startHealth,
			hurtTimer = 0,
			stunTimer = 0,
			facingLeft = true,
			actionTimer = 0,
			attackTimer = 0,
			jumpTimer = 0,
			spin = 0,
			turnTimer = 0,
			delayedFacingLeft = true
			
		})
end

local function walkerMonsterBase(monster, time, isShy, isPulse, isBounce)

	if monster.hurtTimer > 0 then
		monster.hurtTimer = monster.hurtTimer - time
	elseif monster.stunTimer > 0 then
		monster.stunTimer = monster.stunTimer - time
	else
		-- gravity
		if isPulse then
			monster.vy = monster.vy + time * 500
		elseif isBounce then
			monster.vy = monster.vy + time * 1000
		else
			monster.vy = monster.vy + time * 1600
		end
		
		
		local testPos = monster.y + monster.vy * time
		local floorBlocked = gameField.isPositionBlocked(math.ceil(monster.x / 32), math.ceil((testPos+16) / 32)) or gameField.isPositionBlocked(math.ceil(monster.x / 32), math.ceil((testPos-16) / 32))
		if floorBlocked then
			
			--monster.y = testPos - testPos % 32
			if isBounce and monster.vy > 0 then
				monster.vy = -500
				monster.jumpTimer = 1
			else
				monster.vy = 0
			end
		else
			monster.y = testPos
		end
		

		if monster.facingLeft then
			if isPulse then
				if monster.actionTimer <= 0 and math.abs(playerFay.y - monster.y) < 100 then
					monster.actionTimer = 3.5	
				end
				monster.vx = monster.vx - (0.1+monster.actionTimer)*time*160
				monster.vx = monster.vx - monster.vx*time*2
				testPos = monster.x + monster.vx * time - 10	
			elseif isBounce then
				monster.vx = monster.vx + (-100 - monster.vx) * time * 0.5
				testPos = monster.x + monster.vx * time - 10
			else
				monster.vx = monster.vx + (-100 - monster.vx) * time * 0.5
				testPos = monster.x + monster.vx * time - 10
			end
		else
			if isPulse then
				if monster.actionTimer <= 0 and math.abs(playerFay.y - monster.y) < 100 then
					monster.actionTimer = 3.5
				end
				monster.vx = monster.vx + (0.1+monster.actionTimer)*time*160
				monster.vx = monster.vx - monster.vx*time*2
				testPos = monster.x + monster.vx * time + 10		
			elseif isBounce then
				monster.vx = monster.vx + (100 - monster.vx) * time * 0.5
				testPos = monster.x + monster.vx * time + 10		
			else
				monster.vx = monster.vx + (100 - monster.vx) * time * 0.5
				testPos = monster.x + monster.vx * time + 10
			end
		end

		local wallBlocked = gameField.isPositionBlocked(math.ceil(testPos / 32), math.ceil(monster.y / 32))
		local shyCheck = isShy and not gameField.isPositionBlocked(math.ceil(testPos / 32), math.ceil((monster.y + 32) / 32))
		
		local pulseCheck = isPulse and monster.jumpTimer < 1 and not gameField.isPositionBlocked(math.ceil(testPos / 32), math.ceil((monster.y + 32) / 32)) and gameField.isPositionBlocked(math.ceil(monster.x / 32), math.ceil((monster.y + 32) / 32))
		
		if isPulse and monster.actionTimer < 1.2 and monster.facingLeft and playerFay.x > monster.x then
			monster.vx = 0
			monster.facingLeft = not monster.facingLeft
		end
		if isPulse and monster.actionTimer < 1.2 and not monster.facingLeft and playerFay.x < monster.x then
			monster.vx = 0
			monster.facingLeft = not monster.facingLeft
		end
		
		if isPulse and monster.jumpTimer < 1 and playerFay.y < monster.y - 100 and playerFay.y > monster.y - 500 and gameField.isPositionBlocked(math.ceil(monster.x / 32), math.ceil((monster.y + 32) / 32)) then
			monster.vy = -350
			monster.jumpTimer = 3
		end
		
		if wallBlocked or shyCheck then
			monster.vx = 0
			monster.facingLeft = not monster.facingLeft
			monster.turnTimer = 0.5
		elseif pulseCheck then
			monster.x = monster.x + monster.vx * time
			monster.vy = -350
			monster.jumpTimer = 3
		else
			monster.x = monster.x + monster.vx * time
		end
	end
end





local function walkerMonster(monster, time)
	walkerMonsterBase(monster, time, false)
end
local function walkerMonsterShy(monster, time)
	walkerMonsterBase(monster, time, true)
end

local function walkerMonsterPulse(monster, time)
	walkerMonsterBase(monster, time, false, true)
end

local function walkerMonsterJump(monster, time)
	walkerMonsterBase(monster, time, false, false, true)
end


local function flyerWallCheck(monster, time)

	local horizontal,vertical = false, false
	local nextX,nextY = monster.x + monster.vx * time, monster.y + monster.vy * time

	if monster.vx < 0 then
		horizontal = gameField.isPositionBlocked(math.ceil((nextX - 10) / 32), math.ceil(nextY / 32))
	else
		horizontal = gameField.isPositionBlocked(math.ceil((nextX + 10) / 32), math.ceil(nextY / 32))
	end
	if monster.vy < 0 then
		vertical = gameField.isPositionBlocked(math.ceil(nextX / 32), math.ceil((nextY - 10) / 32))
	else
		vertical = gameField.isPositionBlocked(math.ceil(nextX / 32), math.ceil((nextY + 10) / 32))
	end

	return horizontal,vertical
end

local function randomFlyer(monster, time)
	
	if monster.hurtTimer <= 0 and monster.stunTimer <= 0 then
	
		monster.tvx = monster.tvx or (-150 + math.random() * 300)
		monster.tvy = monster.tvy or (-150 + math.random() * 300)
		monster.nextChange = monster.nextChange or math.random() * 10
		monster.nextChange = monster.nextChange - time

		if monster.hurtTimer > 0 and monster.nextChange >0.5 then
		monster.nextChange = 0
		end

		monster.vx = monster.vx + (monster.tvx - monster.vx) * time * .5
		monster.vy = monster.vy + (monster.tvy - monster.vy) * time * .5

		monster.spin = monster.spin + math.sin(monster.tvx+monster.tvy)*0.005

		local h,v = flyerWallCheck(monster, time)
		if h or v or monster.nextChange <= 0 then
			if h then
				monster.vx = 0
			end
			if v then
				monster.vy = 0
			end


			monster.tvx = math.random(50, 250)
			if math.random() > .5 then monster.tvx = -monster.tvx end
			monster.tvy = math.random(50, 250)
			if math.random() > .5 then monster.tvy = -monster.tvy end

			monster.nextChange = .5 + math.random() * 2
		else
			monster.x = monster.x + monster.vx * time
			monster.y = monster.y + monster.vy * time
		end
	end
end

local function medusaFlyer(monster, time)
	if monster.hurtTimer <= 0 and monster.stunTimer <= 0 then
		if not monster.inited then
			monster.inited = true
			monster.ty = monster.y
			if math.random() > .5 then
				monster.vy = -300
			else
				monster.vy = 300
			end
			if math.random() > .5 then
				monster.vx = -100
			else
				monster.vx = 100
			end
		end

		monster.vy = monster.vy + (monster.ty - monster.y) * time * 8

		local h,v = flyerWallCheck(monster, time)
		if h then
			monster.vx = -monster.vx
			monster.facingLeft = monster.vx < 0
		end
		if v then
			monster.vy = -monster.vy
		end
		monster.x = monster.x + monster.vx * time
		monster.y = monster.y + monster.vy * time
	end
end

local function createPowerup(x, y, powerUpChance)
	if math.random() < powerUpChance then
		local playerPowerups = playerPowerups
		local type = math.random(1,15)
		while playerPowerups.getPowerupType(type) == -1 and math.random() > .5 do
			type = math.random(1,15)
		end
		table.insert(powerups, { x=x, y=y, type=type, animationOffset = math.random() * 50 })
	end

end

local function monsterBaseAttack(monster, x, y)
	if monster.attackTimer <= 0 then
		monster.attackTimer = 0.5
	end
end

local function monsterJumpAttack(monster, x, y)
	if monster.attackTimer <= 0 then
		monster.attackTimer = 0.5
		monster.vy = -250
	end
end


local function monsterDeath(monster)
	
	
end

local function medusaDeath(monster)
	addFx(monsterSprite.medusaWing1, monster.x, monster.y, math.random()*400-200 ,
	math.random()*400-200, 1.0, 1.0, 255, 0, math.random()*1+1, math.random()*9.5-4, fxBloodGusher,125)
	
	addFx(monsterSprite.medusaWing2, monster.x, monster.y, math.random()*200-100 ,
	math.random()*400-200, 1.0, 1.0, 255, 0, math.random()*1+1, math.random()*9.5-4, fxBloodGusher,125)
	addFx(monsterSprite.medusaWing3, monster.x, monster.y, math.random()*200-100 ,
	math.random()*400-200, 1.0, 1.0, 255, 0, math.random()*1+1, math.random()*9.5-4, fxBloodGusher,125)
	addFx(monsterSprite.medusaWing4, monster.x, monster.y, math.random()*200-100 ,
	math.random()*200-400, 1.0, 1.0, 255, 0, math.random()*1+1, math.random()*9.5-4, fxBloodGusher,125)
	addFx(monsterSprite.medusaHead, monster.x, monster.y, math.random()*200-100 ,
	math.random()*400-200, 1.0, 1.0, 255, 0, math.random()*1+1, math.random()*9.5-4, fxBloodGusher,125)
	addFx(monsterSprite.medusaJaw, monster.x, monster.y, math.random()*200-100 ,
	math.random()*400-200, 1.0, 1.0, 255, 0, math.random()*1+1, math.random()*9.5-4, fxBloodGusher,125)
	addFx(monsterSprite.medusaBody, monster.x, monster.y, math.random()*200-100 ,
	math.random()*400-200, 1.0, 1.0, 255, 0, math.random()*1+1, math.random()*9.5-4, fxBloodGusher,125)
end

local function wormDeath(monster)
	addFx(monsterSprite.wormPart1, monster.x, monster.y, math.random()*400-200 ,
	math.random()*200-200, 1.0, 1.0, 255, 0, math.random()*1+1, math.random()*9.5-4, fxBloodGusher, 300)
	addFx(monsterSprite.wormPart2, monster.x, monster.y, math.random()*400-200 ,
	math.random()*200-200, 1.0, 1.0, 255, 0, math.random()*1+1, math.random()*9.5-4, fxBloodGusher, 300)
	addFx(monsterSprite.wormPart3, monster.x, monster.y, math.random()*400-200 ,
	math.random()*200-200, 1.0, 1.0, 255, 0, math.random()*1+1, math.random()*9.5-4, fxBloodGusher, 300)
	addFx(monsterSprite.wormPart4, monster.x, monster.y, math.random()*400-200 ,
	math.random()*200-200, 1.0, 1.0, 255, 0, math.random()*1+1, math.random()*9.5-4, fxBloodGusher, 300)
	addFx(monsterSprite.meatPart1, monster.x, monster.y, math.random()*400-200 ,
	math.random()*200-200, 1.0, 1.0, 255, 0, math.random()*1+3, 0, nil, 300)
	addFx(monsterSprite.meatPart2, monster.x, monster.y, math.random()*400-200 ,
	math.random()*200-200, 1.0, 1.0, 255, 0, math.random()*1+3, 0, nil, 300)
	addFx(monsterSprite.meatPart2, monster.x, monster.y, math.random()*400-200 ,
	math.random()*200-200, 1.0, 1.0, 255, 0, math.random()*1+3, 0, nil, 300)
	addFx(monsterSprite.meatPart3, monster.x, monster.y, math.random()*400-200 ,
	math.random()*200-200, 1.0, 1.0, 255, 0, math.random()*1+3, 0, nil, 300)
	addFx(monsterSprite.meatPart4, monster.x, monster.y, math.random()*400-200 ,
	math.random()*200-200, 1.0, 1.0, 255, 0, math.random()*1+3, 0, nil, 300)
	addFx(monsterSprite.meatPart4, monster.x, monster.y, math.random()*400-200 ,
	math.random()*200-200, 1.0, 1.0, 255, 0, math.random()*1+3, 0, nil, 300)
	addFx(monsterSprite.meatPart4, monster.x, monster.y, math.random()*400-200 ,
	math.random()*200-200, 1.0, 1.0, 255, 0, math.random()*1+3, 0, nil, 300)

end

local function shellDeath(monster)
	addFx(monsterSprite.shellBody, monster.x, monster.y, math.random()*400-200 ,
	math.random()*200-200, 1.0, 1.0, 255, 0, math.random()*1+2, math.random()*9.5-4, fxBloodGusher, 300)
	
	for x = 1, 7 do	
		addFx(monsterSprite.shellLeg, monster.x, monster.y, math.random()*400-200 ,
		math.random()*200-200, 1.0, 1.0, 255, 0, math.random()*1+3, math.random()*9.5-4, nil, 300, x)
	end

end

local function klibbDeath(monster)
	addFx(monsterSprite.klibbBody, monster.x, monster.y, math.random()*400-200 ,
	math.random()*200-200, 1.0, 1.0, 255, 0, math.random()*1+1, math.random()*9.5-4, fxBloodGusher, 300)
	
	for x = 1, 7 do	
		addFx(monsterSprite.klibbArm, monster.x, monster.y, math.random()*400-200 ,
		math.random()*200-200, 1.0, 1.0, 255, 0, math.random()*1+3, math.random()*9.5-4, nil, 300, x)
	end

end

local function jumpDeath(monster)
	addFx(monsterSprite.jumpBody, monster.x, monster.y, math.random()*400-200 ,
	math.random()*200-200, 1.0, 1.0, 255, 0, math.random()*1+2, math.random()*9.5-4, fxBloodGusher, 200)
	
	addFx(monsterSprite.jumpLeg, monster.x, monster.y, math.random()*400-200 ,
	math.random()*200-200, 1.0, 1.0, 255, 0, math.random()*1+3, math.random()*9.5-4, nil, 200)
end

local function spikesDeath(monster)
	addFx(monsterSprite.spikesHead, monster.x, monster.y, math.random()*400-200 ,
	math.random()*200-200, 1.0, 1.0, 255, 0, math.random()*1+2, math.random()*9.5-4, fxBloodGusher, 200)
	
	for x = 1, 4 do	
		addFx(monsterSprite.spikesArm, monster.x, monster.y, math.random()*400-200 ,
		math.random()*200-200, 1.0, 1.0, 255, 0, math.random()*1+3, math.random()*4.0-2.0, nil, 200,math.random()*math.pi*2 )
	end
end

function updateMonsters(time)
	for index,monster in pairs(monsters) do
		if monster.active then
			monster.kind.updateMethod(monster, time)

			if math.abs(monster.x - playerFay.x) <= 32 and math.abs(monster.y - playerFay.y) <= 32 then
				
				if monster.stunTimer <= 0 then
				playerFay.hurtMethod(1, monster.x, monster.y)
				monster.kind.attackMethod(monster, playerFay.x, playerFay.y)
				elseif playerFay.vy > 15 then
					playerFay.vy = -playerFay.vy
					if playerFay.jump then
						playerFay.vy = math.min( playerFay.vy - 300, -450)
					else
						playerFay.vy = playerFay.vy + 100
						if playerFay.vy > -500 then
							playerFay.vy = -500
						end
					end
					if playerFay.vy < -1500 then
						playerFay.vy = -1500
					end
					local pitch = 1.0 + (-(playerFay.vy + 400) / 2500.0)
					if pitch > 2.0 then
						pitch = 2.0
					end
					local volume = 0.2 + pitch * 0.4
					daisy.playSound("bounce.wav", volume, 0.0, pitch)

					playerFay.jumpTimer = 0
					playerFay.touchesGround = false
					playerFay.preventBunny = true
				end
				
			end
			if math.abs(monster.y - playerCorn.x) <= 32 and math.abs(monster.x - playerCorn.y) <= 32 then
				playerCorn.hurtMethod(1, monster.x, monster.y)
				monster.kind.attackMethod(monster, playerCorn.x, playerCorn.y)
			end

			for shotIndex,shot in pairs(cornShots) do
				if math.abs(monster.y - shot.x) <= 20 and math.abs(monster.x - shot.y) <= 20 then

					daisy.playSound("EagleExplosion" .. math.random(1,3) .. ".wav")
					
					local wasKilled = false
					monster.hurtTimer = .5
					monster.health = monster.health - 1
					if monster.health <= 0 then
						createPowerup(monster.x, monster.y, monster.kind.powerUpChance)
						monster.kind.deathMethod(monster)
						table.remove(monsters, index)
						wasKilled = true
					end

					notifyCornShotHit(shot.x, shot.y, wasKilled)
					table.remove(cornShots, shotIndex)
				end
			end
			if monster.hurtTimer > 0 then
				monster.hurtTimer = monster.hurtTimer - time
			end
			if monster.stunTimer > 0 then
				monster.stunTimer = monster.stunTimer - time
			end
			if monster.actionTimer > 0 then
				monster.actionTimer = monster.actionTimer - time
			end
			if monster.attackTimer > 0 then
				monster.attackTimer = monster.attackTimer - time
			end
			if monster.jumpTimer > 0 then
				monster.jumpTimer = monster.jumpTimer - time
			end
			if monster.turnTimer > 0 then
				monster.turnTimer = monster.turnTimer - time
			else
				monster.delayedFacingLeft = monster.facingLeft
			end
		else
			if math.abs(monster.y - playerFay.y) < 900 or math.abs(monster.y - playerCorn.x) < 900 then
				monster.active = true
			end
		end
	end
end
local function getColor(monster)
	local r,g,b = 255,255,255
	if monster.hurtTimer > 0 then
		g,b = 0,0
	end
	if monster.stunTimer > 2.5 then
		r,g,b = 0,255,255
	elseif math.sin(monster.stunTimer*18) > 0  then
		r,g,b = 0,255,255
	end
	
	return r,g,b

end
local function renderMonster(monster, view, renderMethod)

	if monster.hurtTimer > 0 then
		renderMethod(view, monsterSprite.eye, monster.x, monster.y, 1, 0, 255, 255, 0, 0)
	else
		renderMethod(view, monsterSprite.eye, monster.x, monster.y, 1, 0, 255, 255, 255, 255)
	end

end

local function renderShell(monster, view, renderMethod)
	local cr,cg,cb = getColor(monster)
	
	for x = 1,7 do
		renderMethod(view, monsterSprite.shellLeg, monster.x+ math.cos((math.pi*2 * x)/7 + monster.x*0.04)*(6+monster.attackTimer*16), monster.y-2+ math.sin((math.pi*2 * x)/7 + monster.x*0.04)*(6+monster.attackTimer*16), 1, (math.pi*2 * x)/7 + monster.x*0.04, 255, cr, cg, cb)
	end

	renderMethod(view, monsterSprite.shellBody, monster.x, monster.y, 1, 0, 255, cr, cg, cb)
		
		
end

local function renderKlibb(monster, view, renderMethod)
	local cr,cg,cb = getColor(monster)
	
	for x = 1,7 do
		renderMethod(view, monsterSprite.klibbArm, monster.x+ math.cos((math.pi*2 * x)/7 - monster.x*0.09)*(12+math.cos(monster.x*0.05 + x)*math.sin(monster.x*0.02)*4), monster.y- math.sin((math.pi*2 * x)/7 - monster.x*0.09)*(12+math.cos(monster.x*0.05+ x)*math.sin(monster.x*0.02)*4), 1, (math.pi*2 * x)/7 - monster.x*0.09, 255, cr, cg, cb)
	end

	renderMethod(view, monsterSprite.klibbBody, monster.x, monster.y-math.cos((math.pi*2) - monster.x*0.05)*3 -1, 1, 0, 255, cr, cg, cb)
		
		
end

local function renderSpikes(monster, view, renderMethod)
	local cr,cg,cb = getColor(monster)
	
	for x = 1,4 do
		renderMethod(view, monsterSprite.spikesArm, monster.x+ math.cos((math.pi*2 * x)/7 + monster.x*0.01*x*0.5)*3, monster.y+2-x+ math.sin((math.pi*2 * x)/7 + monster.x*0.02)*3, 0.5*(1.0+x/10), math.pi*0.5 + math.cos((2+x/4) + monster.x*0.044*x)*0.4, 255, cr, cg, cb)
	end

	renderMethod(view, monsterSprite.spikesHead, monster.x, monster.y-4, 1, math.cos(monster.x*0.124)*0.1, 255, cr, cg, cb)
		
		
end

local function renderBounce(monster, view, renderMethod)
	local cr,cg,cb = getColor(monster)
	renderMethod(view, monsterSprite.jumpLeg, monster.x, monster.y-10+monster.jumpTimer*12, 1, 0, 255, cr, cg, cb)
	renderMethod(view, monsterSprite.jumpBody, monster.x, monster.y-4, 1, 0, 255, cr, cg, cb)
		
		
end

local function renderMedusa(monster, view, renderMethod)
	local cr,cg,cb = getColor(monster)
	local a, r = 0,0 
	
	a,r = monster.spin,0
	renderMethod(view, monsterSprite.medusaBody, monster.x, monster.y, 1, -a, 255, cr, cg, cb)
	r = math.cos(math.max(monster.x+monster.y)*0.23)*1.2
	renderMethod(view, monsterSprite.medusaWing1, monster.x + math.cos(a + 1.8*math.pi)*7, monster.y- math.sin(a + 1.8*math.pi)*7, 1, -a+r, 255, cr, cg, cb)
	r = math.cos(math.max(monster.x+monster.y+234)*0.225)*1.0
	renderMethod(view, monsterSprite.medusaWing2, monster.x + math.cos(a + 0.25*math.pi)*7, monster.y- math.sin(a + 0.25*math.pi)*7, 1, -a+r, 255, cr, cg, cb)
	r = math.cos(math.max(monster.x+monster.y+134)*0.235)*1.3
	renderMethod(view, monsterSprite.medusaWing3, monster.x + math.cos(a + 0.55*math.pi)*7, monster.y- math.sin(a + 0.55*math.pi)*7, 1, -a+r, 255, cr, cg, cb)
	r = math.cos(math.max(monster.x+monster.y+50)*0.229)*0.9
	renderMethod(view, monsterSprite.medusaWing4, monster.x + math.cos(a + 1.00*math.pi)*7, monster.y- math.sin(a + 1.00*math.pi)*7, 1, -a+r, 255, cr, cg, cb)
	
	r = math.abs(math.cos(math.abs(monster.x+monster.y+50)*0.0229)*0.3)*-1
	renderMethod(view, monsterSprite.medusaJaw, monster.x + math.cos(a + 1.35*math.pi)*21, monster.y- math.sin(a + 1.35*math.pi)*21, 1, -a+r, 255, cr, cg, cb)
	r = math.abs(math.cos(math.abs(monster.x+monster.y+50)*0.0629)*0.3)
	renderMethod(view, monsterSprite.medusaHead, monster.x + math.cos(a + 1.30*math.pi)*21, monster.y- math.sin(a + 1.30*math.pi)*21, 1, -a+r, 255, cr, cg, cb)
		
end

local function renderWorm(monster, view, renderMethod)
	local cr,cg,cb = getColor(monster)
	
	local dir = "r"
	local off = -1
	local div = 1
	if monster.delayedFacingLeft and monster.turnTimer > 0.25 then
		dir = "l"
		off = 1
	end
	if monster.turnTimer > 0.25 then
		div = 0.01 + 1/math.cos(math.pi - monster.turnTimer*2 * math.pi)
	end
	if monster.turnTimer <= 0.25 and monster.turnTimer > 0 then
			div = 0.01 + 1/math.cos(monster.turnTimer*2 * math.pi)
	end
	if monster.facingLeft and monster.turnTimer < 0.25 then
		dir = "l"
		off = 1
	end
	
	
	
	renderMethod(view, monsterSprite.wormPart4[dir], monster.x + (math.cos((monster.x*10+250)*0.01)*5+49/div)*off, monster.y+10, 1, 0, 255, cr, cg, cb)
	renderMethod(view, monsterSprite.wormPart3[dir], monster.x + (math.cos((monster.x*10+200)*0.01)*5+40/div)*off, monster.y+9, 1, 0, 255, cr, cg, cb)
	renderMethod(view, monsterSprite.wormPart2[dir], monster.x + (math.cos((monster.x*10+150)*0.01)*5+31/div)*off, monster.y+8, 1, 0, 255, cr, cg, cb)
	renderMethod(view, monsterSprite.wormPart1[dir], monster.x + (math.cos((monster.x*10+50)*0.01)*5+22/div)*off, monster.y+4, 1, 0, 255, cr, cg, cb)
	renderMethod(view, monsterSprite.wormHead[dir], monster.x + (math.cos((monster.x*10)*0.01)*5+0)*off, monster.y-4, 1, 0, 255, cr, cg, cb)
		
end

function initMonsters(field, fay, corn, shots, pows)
	gameField = field
	playerFay = fay
	playerCorn = corn
	cornShots = shots
	powerups = pows

	monsterSprite.circle = daisy.createSpriteState("Circle", "itt.dat")
	monsterSprite.eye = daisy.createSpriteState("EyeMonster", "players.dat")
	monsterSprite.shellBody = daisy.createSpriteState("shellBody", "monsters.dat")
	monsterSprite.shellLeg = daisy.createSpriteState("shellLeg", "monsters.dat")
	monsterSprite.spikesHead = daisy.createSpriteState("spikesHead", "monsters.dat")
	monsterSprite.spikesArm = daisy.createSpriteState("spikesArm", "monsters.dat")
	monsterSprite.klibbBody = daisy.createSpriteState("klibbBody", "monsters.dat")
	monsterSprite.klibbArm = daisy.createSpriteState("klibbArm", "monsters.dat")
	monsterSprite.jumpBody = daisy.createSpriteState("jumpBody", "monsters.dat")
	monsterSprite.jumpLeg = daisy.createSpriteState("jumpLeg", "monsters.dat")
	monsterSprite.meatPart1 = daisy.createSpriteState("meatPart1", "monsters.dat")
	monsterSprite.meatPart2 = daisy.createSpriteState("meatPart2", "monsters.dat")
	monsterSprite.meatPart3 = daisy.createSpriteState("meatPart3", "monsters.dat")
	monsterSprite.meatPart4 = daisy.createSpriteState("meatPart4", "monsters.dat")
	monsterSprite.medusaBody = daisy.createSpriteState("medusaBody", "monsters.dat")
	monsterSprite.medusaHead = daisy.createSpriteState("medusaHead", "monsters.dat")
	monsterSprite.medusaJaw = daisy.createSpriteState("medusaJaw", "monsters.dat")
	monsterSprite.medusaWing1 = daisy.createSpriteState("medusaWing1", "monsters.dat")
	monsterSprite.medusaWing2 = daisy.createSpriteState("medusaWing2", "monsters.dat")
	monsterSprite.medusaWing3 = daisy.createSpriteState("medusaWing3", "monsters.dat")
	monsterSprite.medusaWing4 = daisy.createSpriteState("medusaWing4", "monsters.dat")
	monsterSprite.wormHead = {l = daisy.createSpriteState("wormHeadL", "monsters.dat"), r = daisy.createSpriteState("wormHeadR", "monsters.dat")}
	monsterSprite.wormPart1 = {l = daisy.createSpriteState("wormPart1L", "monsters.dat"), r = daisy.createSpriteState("wormPart1R", "monsters.dat")}
	monsterSprite.wormPart2 = {l = daisy.createSpriteState("wormPart2L", "monsters.dat"), r = daisy.createSpriteState("wormPart2R", "monsters.dat")}
	monsterSprite.wormPart3 = {l = daisy.createSpriteState("wormPart3L", "monsters.dat"), r = daisy.createSpriteState("wormPart3R", "monsters.dat")}
	monsterSprite.wormPart4 = {l = daisy.createSpriteState("wormPart4L", "monsters.dat"), r = daisy.createSpriteState("wormPart4R", "monsters.dat")}
	
	
	addMonsterKind(3, medusaFlyer, renderMonster, monsterBaseAttack, monsterDeath, 0.33)
	addMonsterKind(3, randomFlyer, renderMedusa, monsterBaseAttack, medusaDeath, 0.37)
	addMonsterKind(3, walkerMonster, renderShell, monsterJumpAttack, shellDeath, 0.25)
	addMonsterKind(2, walkerMonsterShy, renderSpikes, monsterJumpAttack, spikesDeath, 0.05)
	addMonsterKind(3, walkerMonsterPulse, renderKlibb, monsterBaseAttack, klibbDeath, 0.37)
	addMonsterKind(3, walkerMonsterJump, renderBounce, monsterBaseAttack,  jumpDeath, 0.33)
	addMonsterKind(5, walkerMonsterShy, renderWorm, monsterBaseAttack, wormDeath, 1.0)

	local height = #(field[1])
	local width = #field

	local x, y = 2, 8
	while y < (height - 15) do

		x = x + math.random(1,width*4)
		while x > (width-1) do
			x = x - (width-2)
			y = y + 1
		end

		if not field.isPositionBlocked(x, y) then
			createMonster(monsterKinds[math.random(#monsterKinds)], x * 32 - 16, y * 32 - 16)
		end

	end
	--print("Num monsters: " .. #monsters)
end

function getMonsters()
	return monsters
end
